-- SageTV.lua
-- Derived from the most excellent RSS reader, Reader.lua used by Evereader.ini and 
-- written by Kaelri

function Initialize()
	-- SET UPDATE DIVIDER
	SKIN:Bang('!SetOption', SELF:GetName(), 'UpdateDivider', -1)
	-- This script never needs to update on a schedule. It should only
	-- update when it gets a "Refresh" command from WebParser.

	-- CREATE MAIN DATABASE
	Feeds = {}

	-- GET MEASURE NAMES
	local AllMeasureNames = SELF:GetOption('MeasureName', '')
	for MeasureName in string.gmatch(AllMeasureNames, '[^%|]+') do
		table.insert(Feeds, {
			Measure     = SKIN:GetMeasure(MeasureName),
			MeasureName = MeasureName,
			Raw         = nil,
			Type        = nil,
			Title       = nil,
			Link        = nil,
			Error       = nil
			})
	end

	-- SET STARTING FEED
	f = f or 1

	-- GET COLORs
	HighlightColor=SKIN:GetVariable('HighlightColor')
	DefaultColor  =SKIN:GetVariable('Color2')

	
end

function Update()
	Input()
	return Output()
end

-----------------------------------------------------------------------
-- INPUT
-- Input() parses the text from WebParser and populates the appropriate feed data.
function Input(a)
	local f = a or f

	local Raw = Feeds[f].Measure:GetStringValue()
	
	if Raw == '' then
		return false
	elseif (Raw ~= Feeds[f].Raw) then
		Feeds[f].Raw = Raw
 
		-- GET NEW DATA
		Feeds[f].Title = SKIN:GetVariable('URL'..f..'Title')
		Feeds[f].Link  = string.match(Raw, '<link.->(.-)</link>')          or nil

		local Items = {}
		for Item in string.gmatch(Raw, '<item.-</item>') do

			local Title = string.match(Item, '<title.->(.-)</title>' ) or 'Untitled'

			local Link  = string.match(Item, '<link.->http://(.-)</link>')      	or nil
			local ID    = string.match(Item, '<guid.->(.-)</guid>')         		or Link or Title
			
			local Date  = string.match(Title, '.*at(.-),.-on')	   or nil
			local Date  = Date .. ' ' .. string.match(Title, ', [0-9][0-9][0-9][0-9] (.*) on')
			local Date  = string.match(Item, '<pubDate.->(.-),.--') .. ',' .. Date	or nil



			local Desc  = string.match(Item, '<description.->(.-)</description>')   or nil
			local Desc  = Desc and string.gsub(Desc, '<.->',       '') or nil
			local Desc  = Desc and string.gsub(Desc, '&lt;.-&gt;', '') or nil

			local Channel = string.match(Title, '[^"]+.- on (.*)') or "Unknown"
			
			local Title = string.match(Title, '([^"]+)')

			local Color=DefaultColor
			local Recording = string.match(ID,'.*MediaFileId=(.*)') or False
			if Recording then 
				Color=HighlightColor
			end

			table.insert(Items, {
				ID      = ID,
				Title   = Title,
				Link    = Link,
				Desc    = Desc,
				Date    = Date,
				Channel = Channel,
				Color = Color
				})

		end

		-- ADD NEW ITEMS
		for i = #Items, 1, -1 do
			if Items[i] then
				table.insert(Feeds[f], 1, Items[i])
			end
		end

		-- CHECK NUMBER OF ITEMS
		local MaxItems = SELF:GetNumberOption('ItemsToShow', nil)
		local MaxItems = (MaxItems > 0) and MaxItems or nil

		if #Feeds[f] == 0 then
			Feeds[f].Error = {
				Description = 'No items found.',
				Title       = Feeds[f]['Title'],
				Link        = Feeds[f]['Link']
			}
			return false
		elseif MaxItems and (#Feeds[f] > MaxItems) then
			for i = #Feeds[f], (MaxItems + 1), -1 do
				table.remove(Feeds[f])
			end
		end

		-- CLEAR ERRORS FROM PREVIOUS UPDATE
		Feeds[f].Error = nil

	end

	return true
end

-----------------------------------------------------------------------
-- OUTPUT
-- Output() updates the meters with the data in the feed.
function Output()
	local Queue = {}
	local Feed  = Feeds[f]

	-- BUILD QUEUE
	Queue['CurrentFeed']   = f
	Queue['NumberOfItems'] = #Feed

	local ItemsToShow  = SELF:GetNumberOption('ItemsToShow', 0)

	Queue['FeedTitle'] = Feed.Title
	Queue['FeedLink']  = Feed.Link or ''

	for i = 1, math.max(#Feed, ItemsToShow) do
		local Item = Feed[i] or {}			
		Queue['Item'..i..'Title']   = Item.Title   or ''
		Queue['Item'..i..'Link']    = Item.Link    or Feed.Link or ''
		Queue['Item'..i..'Desc']    = Item.Desc    or ''
		Queue['Item'..i..'Date']    = Item.Date    or ''
		Queue['Item'..i..'Color']  = Item.Color  or DefaultColor
		Queue['Item'..i..'Channel'] = Item.Channel or ''
	end

	-- Hide the airings
	local VariablePrefix = SELF:GetOption('VariablePrefix', '')
	for i=1, 9 do  -- 9 is the maximum items to show
		SKIN:Bang('!HideMeter', VariablePrefix..'Item'..i..'Highlight')
		SKIN:Bang('!HideMeter', VariablePrefix..'Item'..i)
		SKIN:Bang('!HideMeter', VariablePrefix..'Item'..i..'Date')
		SKIN:Bang('!HideMeter', VariablePrefix..'Item'..i..'Chan')
	end

	-- set variables
	for k, v in pairs(Queue) do
		SKIN:Bang('!SetVariable', VariablePrefix..k, v)
	end
		
	-- show airings
	for i = 1, ItemsToShow do
		SKIN:Bang('!ShowMeter', VariablePrefix..'Item'..i..'Highlight')
		SKIN:Bang('!ShowMeter', VariablePrefix..'Item'..i)
		SKIN:Bang('!ShowMeter', VariablePrefix..'Item'..i..'Date')
		SKIN:Bang('!ShowMeter', VariablePrefix..'Item'..i..'Chan')	
	end
	
	-- FINISH ACTION   
	local FinishAction = SELF:GetOption('FinishAction', '')
	if FinishAction ~= '' then
		SKIN:Bang(FinishAction)
	end

	return true
end

-----------------------------------------------------------------------
-- EXTERNAL COMMANDS

function Refresh(a)
	a = a and tonumber(a) or f
	if a == f then
		SKIN:Bang('!UpdateMeasure', SELF:GetName())
	else
		Input(a)
	end
end

function Show(a)
	f = tonumber(a)
	SKIN:Bang('!UpdateMeasure', SELF:GetName())
end

function ShowNext()
	f = (f % #Feeds) + 1
	SKIN:Bang('!UpdateMeasure', SELF:GetName())
end

function ShowPrevious()
	f = (f == 1) and #Feeds or (f - 1)
	SKIN:Bang('!UpdateMeasure', SELF:GetName())
end
