function Initialize()
	-- SET UPDATE DIVIDER
	SKIN:Bang('!SetOption', SELF:GetName(), 'UpdateDivider', -1)
	-- This script never needs to update on a schedule. It should only
	-- update when it gets a "Refresh" command from WebParser.
	
	AlertStatus = 0			-- alert level
	UnresolvedConflicts = 0	-- flag indicating if there are unresolved recording conflicts
	RecordingStatus = 0		-- flag indicating if system is currently recording something
	
	WriteStatus = SELF:GetNumberOption('WriteStatus', 0)
	
	StatusFile_Initialize()
end

function Update()
	return alert
end

-----------------------------------------------------------------------
-- Recording() checks to see if there is a recording in progress, and also checks the alert level.
function Recording()
	local Raw = SKIN:GetMeasure('RecordingStatus'):GetStringValue()
	local show

	if Raw == '' then
		return false
	else
		alert = string.match(Raw, '<alertLevel description=".-" level="(.-)"/>' ) or 0
		show = string.match(Raw, '<showList.->(.-)</showList>' ) or nil
	end

	alert=tonumber(alert)

	-- Update the RecordingImage
	local Image = SKIN:GetVariable('RecordingIconOFF')
	RecordingStatus = 0
	if show~=nil then
		Image = SKIN:GetVariable('RecordingIconON')
		RecordingStatus = 1 -- set status flag
	end
	SKIN:Bang('!SetVariable', 'RecordingImage', Image)
	
	AlertStatus=alert
	-- Update the AlertImage
	Image = SKIN:GetVariable('AlertLevelImage0')
	if alert==1 then
		Image = SKIN:GetVariable('AlertLevelImage1')
	elseif alert==2 then
		Image = SKIN:GetVariable('AlertLevelImage2')
	elseif alert==3 then
		Image = SKIN:GetVariable('AlertLevelImage3')
	end
	SKIN:Bang('!SetVariable', 'AlertImage', Image)

	-- note that we don't need to call StatusFile_Update() for both Recording() and Memory()
	
	return
end

-- Memory() determines the used and available disk space, and also checks if there are any unresolved recording conflicts.
function Memory()

	local Raw = SKIN:GetMeasure('SystemStatus'):GetStringValue()
	local available
	local used
	local conflicts

	if Raw == '' then
		return false
	else
		available = string.match(Raw, '.-Available: (.-)... GB.-' ) or 'bad match'
		used = string.match(Raw, '.-Used: (.-)... GB.-') or 'bad match'
		conflicts = string.match(Raw, '.-Unresolved Conflict.-> (.-) Unresolved Conflicts.*') or 'No match'
	end
	
	-- Update the Available memory
	SKIN:Bang('!SetVariable', 'DiskSpaceFree', tonumber(available))
	SKIN:Bang('!SetVariable', 'DiskSpaceUsed', tonumber(used))
	SKIN:Bang('!SetVariable', 'DiskSpaceUsed%', tonumber(used)/(tonumber(used) + tonumber(available)))
	
	-- Update the ConflictsImage
	local Image = SKIN:GetVariable('NoConflict')
	UnresolvedConflicts=0
	if conflicts~='No match' then
		Image = SKIN:GetVariable('ConflictIcon')
		UnresolvedConflicts=tonumber(conflicts) -- set the status flag
	end
	SKIN:Bang('!SetVariable', 'ConflictImage', Image)

	StatusFile_Update()
	
	return
end

-----------------------------------------------------------------------
-- EXTERNAL COMMANDS
-- The Refresh() function is called whenever one of several measures retrieves information from the server.
-- Each type of information is different and needs to be parsed in a different way, so the Refresh() function
-- dispaches control to the appropriate handler.
-- The argument a is a string value containing the type of indicator to be refreshed.
function Refresh(a)
	if a=='Recording' then
		return Recording()
	elseif a=='Memory' then
		return Memory()
	end
end

function StatusFile_Initialize()
	-- DETERMINE FILEPATH
	StatusFile = SELF:GetOption('StatusFile', SELF:GetName()..'Status.xml')
	StatusFile = SKIN:MakePathAbsolute(StatusFile)

	-- CHECK IF FILE EXISTS, but there's no reason to actually read it... the contents will be determined by
	-- querying the SageTV server.
	local ReadFile = io.open(StatusFile)
	if ReadFile then
		ReadFile:close()
	end
	
	-- If WriteStatus=0, we will write 0s to the file.
	if WriteStatus == 0 then
		-- WRITE XML TO FILE
		local WriteFile = io.open(StatusFile, 'w')
		if WriteFile then
			local WriteContent = '<RecordingStatus>0</>\n<AlertStatus>0</>\n<UnresolvedConflicts>0</>'
			WriteFile:write(WriteContent)
			WriteFile:close()
		end
	end

end

function StatusFile_Update()
	if WriteStatus == 1 then
		-- WRITE XML TO FILE
		local WriteFile = io.open(StatusFile, 'w')
		if WriteFile then
			local WriteContent = '<RecordingStatus>'..RecordingStatus..'</>\n<AlertStatus>'..AlertStatus..'</>\n<UnresolvedConflicts>'..UnresolvedConflicts..'</>'
			WriteFile:write(WriteContent)
			WriteFile:close()
		end
	end
end

function ClearStatus()
	if StatusFile then
		os.remove(StatusFile)
	end
end